/**************************************************************************
 * CLASS:		MiscDragView
 *
 *	See the header file for more information on this class.
 *
 * This object is included in the MiscKit by permission from the author
 * and its use is governed by the MiscKit license, found in the file
 * "LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
 * for a list of all applicable permissions and restrictions.
 **************************************************************************/
 
#import <daymisckit/DAYString.h>
#import "MiscDragView.h"


@implementation MiscDragView

- initFrame: (const NXRect *)frameRect
{
    [super initFrame: frameRect];

	[self setDelegate: nil];
	[self setAllowSourceDragging: YES];
	[self setAllowDestinationDragging: YES];
	[self setAcceptForeignDrag: YES];
	[self setAcceptLocalDrag: YES];
	[self setAcceptSelfDrag: YES];
	[self setRetainData: YES];
	dragImage = nil;
		
    return self;
}


// Options that could apply to any subclassed DragView

- setAllowSourceDragging: (BOOL)aBool
{
	allowSourceDragging = aBool;
	
	return self;
}


- (BOOL)allowSourceDragging
{
	return allowSourceDragging;
}


- setAllowDestinationDragging: (BOOL)aBool
{
	allowDestinationDragging = aBool;
	return self;
}


- (BOOL)allowDestinationDragging
{
	return allowDestinationDragging;
}


- setAcceptForeignDrag: (BOOL)aBool
{
	acceptForeignDrag = aBool;
	return self;
}


- (BOOL)acceptForeignDrag
{
	return acceptForeignDrag;
}


- setAcceptLocalDrag: (BOOL)aBool
{
	acceptLocalDrag = aBool;
	return self;
}


- (BOOL)acceptLocalDrag
{
	return acceptLocalDrag;
}


- setAcceptSelfDrag: (BOOL)aBool
{
	acceptSelfDrag = aBool;
	return self;
}


- (BOOL)acceptSelfDrag
{
	return acceptSelfDrag;
}


- setRetainData: (BOOL)aBool
{
	retainData = aBool;
	return self;
}


- (BOOL)retainData
{
	return retainData;
}


// Methods to be the source of a dragging operation 


- mouseDown: (NXEvent *)theEvent
{
  NXPoint  zero = {0.0, 0.0};
  NXPoint  offset = theEvent->location;
  BOOL slide = YES;
  id  pasteboard = [Pasteboard newName: NXDragPboard];
  
 	if ([self allowSourceDragging] == NO)
		return [super mouseDown: theEvent]; 
		 
  	[self convertPoint: &offset fromView: nil]; 
		
    [NXApp preventWindowOrdering];

	// if the overridden setupForSourceDrag returns yes, continue
	
	if ([self setupForSourceDrag])
	{	
		// give the chance to change the offset and dragImage start point
		
		[self calculateDragPoint: &offset andOffset: &zero];
		
		if (dragImage != nil)
		{			
			if ([self retainData] == NO)
				slide = NO;

			// send out the delegate message
			
			[self sourceDragInitiated: self];
							
			// begin the drag session
			
			[self dragImage: dragImage at: &offset offset: &zero 
					event: theEvent	pasteboard: pasteboard 
					source: self slideBack: slide];			
		 }
	 }
	  	
    return self;
}


// This method should be overridden if you want to have some control
// on where the dragged image is first placed, and how far it is offset
// from the original mousedown (dragPoint). As you can see, the original
// does nothing with the dragPoint or offset.

- calculateDragPoint: (NXPoint *)dragPoint andOffset: (NXPoint *)offset
{
	return self;
}


// A class that should be overridden in the subclass to put the information
// on the drag pasteboard (NXDragPboard) and select an image to drag. If you
// do not override this method, then source dragging will not happen.

- (BOOL)setupForSourceDrag
{
	return NO;
}


// The following are methods from the NXSourceDraggingProtocol that are
// here so they are easy to lookup.

- (NXDragOperation)draggingSourceOperationMaskForLocal:(BOOL)flag
{
    return NX_DragOperationAll;
}


- draggedImage:(NXImage *)image beganAt:(NXPoint *)screenPoint
{
	return self;
}


// Sends a "sourceDragFinished:" to the delegate if it responds

- draggedImage:(NXImage *)image endedAt:(NXPoint *)screenPoint 
		deposited:(BOOL)flag
{
	[self sourceDragFinished: flag];
	return self;
}

// Methods to be the destination of a dragging operation 

- (NXDragOperation)draggingEntered: sender
{
	if ([self allowDestinationDragging] == NO)
		return NX_DragOperationNone;
		
 	if ([self acceptSelfDrag] == NO && [sender draggingSource] == self)
		return NX_DragOperationNone;
		
	if ([self acceptForeignDrag] == NO && [sender isDraggingSourceLocal] == NO)
		return NX_DragOperationNone;
		
    if ([self acceptLocalDrag] == NO && [sender isDraggingSourceLocal] == YES)
		return NX_DragOperationNone;
	
	// Send a "destinationDragInitiated:" to the delegate
	
	[self destinationDragInitiated: self];
	
	return NX_DragOperationCopy;
}


- (NXDragOperation)draggingUpdated: sender
{	
	return [super draggingUpdated: sender]; 
}


// Destination drag came to an end, but was not successful, so send a NO
// to the delegate

- draggingExited:sender
{
	[self destinationDragFinished: NO];
    return self;
}


- (BOOL)prepareForDragOperation:sender
{
    return YES;
}


- (BOOL)performDragOperation:sender
{	
    return YES;
}


// Send the delegate a "destinationDragFinished" message

- concludeDragOperation:sender
{
	[self destinationDragFinished: YES];
    return self;
}


// Methods to display, and other useful stuff

- (BOOL)acceptsFirstMouse
{
    return YES;
}


- (BOOL)shouldDelayWindowOrderingForEvent:(NXEvent *)theEvent
{
    return YES;
}

 
// delegate methods
 
- delegate
{
    return delegate;
}


- setDelegate: (id)theDelegate
{
    delegate = theDelegate;
	return self;
}


// Sent from "mouseDown" just before the dragging starts to take place

- sourceDragInitiated: view
{
	if (!delegate)
		return self;
		
	if ([delegate respondsTo: @selector(sourceDragInitiated:)])
		[delegate sourceDragInitiated: view];
	return self;
}


// Sent when "draggedImage: endedAt: screenPoint: deposited:" is given 
// control and success is whether the image was deposited in another
// view somewhere

- sourceDragFinished: (BOOL)success
{
	if (!delegate)
		return self;
		
	if ([delegate respondsTo: @selector(sourceDragFinished:)])
		[delegate sourceDragFinished: success];
	return self;
}


// Sent when "draggingEntered:" is given control

- destinationDragInitiated: view
{
	if (!delegate)
		return self;
		
	if ([delegate respondsTo: @selector(destinationDragInitiated:)])
		[delegate destinationDragInitiated: view];
	return self;
}


// Sent when "concludeDragOperation:" is given control

- destinationDragFinished: (BOOL)success
{
	if (!delegate)
		return self;
		
    if ([delegate respondsTo: @selector(destinationDragFinished:)])
		[delegate destinationDragFinished: success];
	return self;
}


// Archiving methods

- read: (NXTypedStream *)stream
{
	[super read: stream];

	delegate = NXReadObject (stream);
	dragImage = NXReadObject (stream);
	NXReadTypes (stream, "c", &allowSourceDragging);
	NXReadTypes (stream, "c", &allowDestinationDragging);
	NXReadTypes (stream, "cc", &acceptForeignDrag, &acceptLocalDrag);
	NXReadTypes (stream, "cc", &acceptSelfDrag, &retainData);
	
	return self;
}


- write: (NXTypedStream *)stream
{
	[super write: stream];

	NXWriteObjectReference (stream, delegate);
	NXWriteObjectReference (stream, dragImage);
	NXWriteTypes (stream, "c", &allowSourceDragging);
	NXWriteTypes (stream, "c", &allowDestinationDragging);
	NXWriteTypes (stream, "cc", &acceptForeignDrag, &acceptLocalDrag);
	NXWriteTypes (stream, "cc", &acceptSelfDrag, &retainData);
	
	return self;
}



@end
