/***************************************************************************
 * CLASS:		MiscIconWell
 *
 *	See the header file for more information.
 *
 * This object is included in the MiscKit by permission from the author
 * and its use is governed by the MiscKit license, found in the file
 * "LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
 * for a list of all applicable permissions and restrictions.
 ***************************************************************************/

#import <daymisckit/DAYString.h>
#import "MiscIconWell.h"

 
@implementation MiscIconWell
 
- initFrame: (const NXRect *)frameRect
{
  const char *const types[] = {NXFilenamePboardType};
  
  	[super initFrame: frameRect];

	[self registerForDraggedTypes: (const char *const *)&types count: 1];
	
	filename = [ [DAYString alloc] init];
	[self setAllowDoubleClickLaunch: YES];
	
	return self;
}


// I believe you have to registerForDraggedTypes in here also, when the
// view is unarchived everything will work alright.

- awake
{ 
  const char *const types[] = {NXFilenamePboardType};
  
	[super awake];	

	[self registerForDraggedTypes: (const char *const *)&types count: 1];
	if (filename == nil)
		filename = [ [DAYString alloc] init];
	
	return self;
}


- free
{
	if (filename)
		[filename free];
	return [super free];
}


// Return the filename associated with the image. Kind of a hack to return
// nil if no filename set, I suppose. I will fix the way that this entire
// class deals with the filename for the next release.

- (char *)filename
{   
 	if (strcmp([filename stringValue], "") == 0)
		return NULL;
	else
  		return ((char *)[filename stringValue]);
}


// Overridden from ImageView so the icon representation of the file
// is fetched instead of the image itself (likely the filename is not
// an image). It also has the nice hack that if there are more than one
// filename that it steals that multiple.tiff from Librarian.app. Bad, i know.

- setImageByFilename: (char *)aFilename
{
	[filename setStringValue: aFilename];
		
	if ([filename numWords] > 1)
		[super setImageByFilename: "/NextApps/Librarian.app/multiple.tiff"];
	else	
		[self setImage: [ [Application workspace] getIconForFile: aFilename] ];
	
	return self;
}


// Add additional options that only apply to FileDragView

- setAllowDoubleClickLaunch: (BOOL)aBool
{
	allowDoubleClickLaunch = aBool;
	return self;
}


- (BOOL)allowDoubleClickLaunch
{
	return allowDoubleClickLaunch;
}


// Override mouseDown to check if the icon was double clicked. If so, then
// launch it from workspace, else let super handle it.

- mouseDown: (NXEvent *)theEvent
{
	if ([self allowDoubleClickLaunch] && theEvent->data.mouse.click > 1 && 
			filename)
		[ [Application workspace] openFile: [filename stringValue] ];
	else
		[super mouseDown: theEvent];

	return self;
}


// Make the dragPoint be the middle of the image, so it looks nice. This
// method is from MiscDragView, so the subclasses have some control
// on where the image is grabbed in relation to the cursor and mouseDown
// location.

- calculateDragPoint: (NXPoint *)dragPoint andOffset: (NXPoint *)offset
{
	dragPoint->x -= ivSize.width/2.0;
    dragPoint->y -= ivSize.width/2.0;

	return self;
}

	
// Put the data on the pasteboard when a source drag is about to begin, 
// and also choose the image to drag. 

- (BOOL)setupForSourceDrag
{
  id  dragPB = [Pasteboard newName: NXDragPboard];
  
	if (filename != nil)
	{
		[dragPB declareTypes:&NXFilenamePboardType num:1 owner:self];

 		[dragPB writeType: NXFilenamePboard data: [filename stringValue]
		 			length: [filename length] ];
		dragImage = ivImage;

		if ([self retainData] == NO)
		{	ivImage = nil;
			[self display];
		 }
		return YES;
	 }

	return NO;
}


// Check if an incoming dragged icon is using the NXFilenamePboardType. If
// not, then don't accept the drag.

- (BOOL)performDragOperation: sender
{
  id  dragPB = [Pasteboard newName: NXDragPboard];
  char  *pbData;
  int  pbLength;

	if ([dragPB readType: NXFilenamePboardType data: &pbData 
			length: &pbLength] == nil)
		return NO;	
	 
	return YES;
}


// Take the data from the pasteboard and set the new image.

- concludeDragOperation: sender
{
  id  dragPB = [Pasteboard newName: NXDragPboard];
  char  *pbData;
  int  pbLength;
  
	[dragPB readType: NXFilenamePboardType data: &pbData 
			length: &pbLength];

	[self setImageByFilename: pbData];
			
	[dragPB deallocatePasteboardData: pbData length: pbLength];
	
	[super concludeDragOperation: sender];
	
    return self;
}


// Archiving methods

- read: (NXTypedStream *)stream
{
	[super read: stream];

	filename = NXReadObject (stream);
	NXReadTypes (stream, "c", &allowDoubleClickLaunch);

	return self;
}


- write: (NXTypedStream *)stream
{
	[super write: stream];

	NXWriteObject (stream, filename);
	NXWriteTypes (stream, "c", &allowDoubleClickLaunch);

	return self;
}


// Used to tell the IB which Inspector to use for the
// MiscIconWell object. (Used only for palettes)

- (const char *)getInspectorClassName
{
	return "MIWInspector";
}

@end