/* MiscSwapViewByObject.m				 
 *
 * This is a class can handle swapping of different subview (controlled by
 * MiscSwapSubiewController's) into ourself by comparing trigger objects.
 *
 * For more interface-info see the header file. More in depth information
 * can be found here in the source-code.
 *
 * Written by: 		Thomas Engel
 * Created:    		24.01.1994 (Copyleft)
 * Last Modified: 	24.01.1994
 */

#import "MiscSwapViewByObject.h"
#import "MiscSwapSubviewController.h"

@implementation MiscSwapViewByObject
 
- initFrame:(const NXRect *)frameRect
{
	// Normally we try the tags if there are any for the objects.
 	// If you don't like this behavier create your own subclass with different
	// init method or just set it to NO after the init.

	self = [super initFrame:frameRect];
	if( !self ) return self;

	// OK. We really are an object...here we go with our init.

	controllers = [List new];	 
 	[self setTagComparison:YES];
	return self;
}
 
- free
{
	// We do not free the subviewControllers of this list! Well we didin't
	// allocate them..but maybe we should free them...hmmm. I'm not sure yet.
	
	[controllers free];
	return [super free];
}

- trigger
{
	return trigger;
}
 
- swapAction:sender
{	
	// This is the method that has to be triggered by each trigger for a nice
	// swap. It incorporates the delegate and the new&previous viewCtrl.
	// Our delegate gets informed before the viewControllers are.
	// The new controller will get willSwapIn which should cause a revert or
	// any other method causing the contoller to update/init its view. It's
	// the last chance before it will be displayed.
	
	id	oldController;
	
	trigger = sender;

	if( [delegate respondsTo:@selector(viewWillSwap:)] )
		[delegate viewWillSwap:self];

	oldController = currentController;
	currentController = [self findController];

	if( currentController != oldController )
	{
		[oldController willSwapOut];
		[currentController willSwapIn];
	}

	[self setSubview:[currentController view]];
	
	if( [delegate respondsTo:@selector(viewDidSwap:)] )
		[delegate viewDidSwap:self];
	if( currentController != oldController )
	{
		[oldController didSwapOut];
		[currentController didSwapIn];
	}
	
	return self;
}

- addController:sender
{
	// Adding a viewController has to ensure that there is only one controller
	// for one trigger. The controller added last will be the only known
	// controller after adding has finished!
	// To find interferring controllers we use our findController method.
	// This needs the trigger to check and the right tagComparison setting.
	// If you change the tagComparison setting later the results might not be
	// correct!
	
	id	oldTrigger;
	id	oldController;
	
	oldTrigger = trigger;
	trigger = [sender trigger];
	
	// If there already is someone...remove him.!
	
	oldController = [self findController];
	if( oldController ) [self removeController:oldController];
	
	[controllers addObject:sender];
	
	trigger = oldTrigger;
	return self;
}

- removeController:sender
{
	// Here we remove  aViewController from the subviewControllers list.
 	// ATTENTION: This does not cause the view to disappear when it is the
 	// current swapView contents!
 
	[controllers removeObject:sender];
	return self;
}

- removeAllControllers
{
	[controllers empty];
	return self;
}

- controllers
{
	return controllers;
}

- currentController
{
	return currentController;
}

- setTagComparison:(BOOL)flag
{
	// Should we compare the tags first...and then the objects ?
	
	tagComparison = flag;
	return self;
}

- (BOOL)doesTagComparison
{
	return tagComparison;
}

- findController
{
 	// This is the basic comparison center. Subclasses of this class should
	// implement only the new findControllerByTag(Object) methods and leave
	// this method untouched. Sometime there is a way of finding a more
	// 'useable' trigger even inside the swapAction method.

	id	newController;

	newController = nil;	
	
	if( tagComparison ) newController = [self findControllerByTag];
	if( !newController ) newController = [self findControllerByObject];

	return newController;
}

- findControllerByTag
{
	// Here we try to find the right controller by comparing the tags. 
	// Really simple. Well...the trigger has to have a tag different from
	// zero.
	
	id		aViewController;
	int		i;
	int		refTag = 0;
	
	if( [trigger respondsTo:@selector(tag)] )
		refTag = [trigger tag];
		
	if( refTag == 0 ) return nil;
	
	aViewController = nil;	
	
	// Ok now lets find out what viewController refers to this tag.
	
	for( i=0; i<[controllers count]; i++ )
	{
		if( refTag == [[controllers objectAt:i] triggerTag] )
		{
			aViewController = [controllers objectAt:i];
			break;
		};
	}
	return aViewController;
}

- findControllerByObject
{
	// Here we simple compare the objects. They have to be the SAME...not only
	// similar!!!
	
	id		aViewController;
	int		i;
	
	aViewController = nil;	
	
	for( i=0; i<[controllers count]; i++ )
	{
		if( trigger == [[controllers objectAt:i] trigger] )
		{
			aViewController = [controllers objectAt:i];
			break;
		};
	}
	return aViewController;
}

@end

/*
 * History: 24.01.94 Made it a subclass of MiscSwapView
 *
 *			08.01.94 Switched to tagComparison for better reading.
 *					 choosesByTagFirst was not that nice.
 *
 *			21.12.93 Code transferred from the old swapPopManager and
 *					 some viewController methods added plus the trigger object
 *					 which now stores the object triggering our swap.
 *
 *			20.12.93 Enlightened the controller to check the tags if they are
 *					 set. This helps to localize apps.
 *
 *			04.12.93 Added a delegate to this class the enable better 
 *					 command-key handling.
 *
 *			04.11.93 First steps to a general-purpose swapPopManager.
 *
 *
 * Bugs: - I'm not sure about what to free....
 */